@@ -1,5 +1,4 @@
 """Repair Database Integrity Errors."""
-import os
 import re
 import sqlite3
 
@@ -7,35 +6,44 @@ from django.apps import apps
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
 from django.contrib.sessions.models import Session
+from django.db import DEFAULT_DB_ALIAS, connection, connections
+from django.db.migrations.executor import MigrationExecutor
 from django.db.migrations.recorder import MigrationRecorder
 from django.db.models.functions import Now
 from django.db.utils import OperationalError
 
-from codex.settings.logging import get_logger
-from codex.settings.settings import CONFIG_PATH, DB_PATH
+from codex.logger.logging import get_logger
+from codex.settings.settings import (
+    CODEX_PATH,
+    CONFIG_PATH,
+    DB_PATH,
+    SKIP_INTEGRITY_CHECK,
+)
 
-
-NO_0005_ARG = "no_0005"
-OK_EXC_ARGS = (NO_0005_ARG, "no such table: django_migrations")
+NO_MIGRATIONS_TABLE_ERROR = "no such table: django_migrations"
 REPAIR_FLAG_PATH = CONFIG_PATH / "rebuild_db"
 DUMP_LINE_MATCHER = re.compile("TRANSACTION|ROLLBACK|COMMIT")
 REBUILT_DB_PATH = DB_PATH.parent / (DB_PATH.name + ".rebuilt")
-BACKUP_DB_PATH = DB_PATH.parent / (DB_PATH.name + ".backup")
+BACKUP_DB_PATH = DB_PATH.parent / (DB_PATH.name + ".bak")
 MIGRATION_0005 = "0005_auto_20200918_0146"
 MIGRATION_0007 = "0007_auto_20211210_1710"
 MIGRATION_0010 = "0010_haystack"
 MIGRATION_0011 = "0010_library_groups_and_metadata_changes"
+MIGRATION_0018 = "0018_rename_userbookmark_bookmark"
+MIGRATION_0023 = "0023_rename_credit_creator_and_more"
+MIGRATION_0025 = "0025_add_story_arc_number"
 M2M_NAMES = {
     "Character": "characters",
-    "Credit": "credits",
+    "Creator": "creators",
     "Genre": "genres",
     "Location": "locations",
     "SeriesGroup": "series_groups",
-    "StoryArc": "story_arcs",
+    "StoryArcNumber": "story_arc_numbers",
     "Tag": "tags",
     "Team": "teams",
     "Folder": "folders",
 }
+SKIP_M2M_CHECKS = {"Creator": MIGRATION_0023, "StoryArcNumber": MIGRATION_0025}
 NULL_SET = frozenset([None])
 HAVE_LIBRARY_FKS = ("FailedImport", "Folder", "Comic")
 GROUP_HOSTS = {
@@ -45,12 +53,13 @@ GROUP_HOSTS = {
     "Comic": ("Publisher", "Imprint", "Series", "Volume"),
 }
 WATCHED_PATHS = ("Comic", "Folder")
-CREDIT_FIELDS = {"CreditRole": "role", "CreditPerson": "person"}
+CREATOR_FIELDS = {"CreatorRole": "role", "CreatorPerson": "person"}
 OPERATIONAL_ERRORS_BEFORE_MIGRATIONS = (
     "no such column: codex_comic.name",
     "no such column: codex_comic.stat",
     "no such column: codex_folder.stat",
     "no such table: codex_comic_folders",
+    # "no such table: codex_comic_creators",
     "'QuerySet' object has no attribute 'stat'",
 )
 DELETE_BAD_COMIC_FOLDER_RELATIONS_SQL = (
@@ -60,6 +69,8 @@ DELETE_BAD_COMIC_FOLDER_RELATIONS_SQL = (
     'OR NOT ("codex_comic_folder"."folder_id" '
     'IN (SELECT "codex_folder"."id" FROM "codex_folder")))'
 )
+MIGRATION_DIR = CODEX_PATH / "migrations"
+
 LOG = get_logger(__name__)
 
 
@@ -70,15 +81,31 @@ def has_applied_migration(migration_name):
     ).exists()
 
 
-def _repair_old_comic_folder_fks():
+def has_unapplied_migrations():
+    """Check if any migrations are outstanding."""
+    try:
+        connection = connections[DEFAULT_DB_ALIAS]
+        connection.prepare_database()
+        executor = MigrationExecutor(connection)
+        targets = [
+            key for key in executor.loader.graph.leaf_nodes() if key[0] == "codex"
+        ]
+        plan = executor.migration_plan(targets)
+    except Exception as exc:
+        LOG.warning("has_unapplied_migrations()", exc)
+        return False
+    else:
+        return bool(plan)
+
+
+def _delete_old_comic_folder_fks():
     if has_applied_migration(MIGRATION_0007):
         return
-    from django.db import connection
 
     try:
         with connection.cursor() as cursor:
             cursor.execute(DELETE_BAD_COMIC_FOLDER_RELATIONS_SQL)
-        LOG.verbose("Deleted old comic_folder relations with bad integrity.")
+        LOG.info("Deleted old comic_folder relations with bad integrity.")
     except Exception as exc:
         LOG.exception(exc)
 
@@ -88,7 +115,7 @@ def _fix_comic_m2m_integrity_errors(apps, comic_model, m2m_model_name, field_nam
     """Delete relations to comics that don't exist and m2ms that don't exist."""
     m2m_model = apps.get_model("codex", m2m_model_name)
     field = getattr(comic_model, field_name)
-    through_model = field.through  # noqa: N806
+    through_model = field.through
     link_col = field_name[:-1].replace("_", "") + "_id"
 
     m2m_rels_with_bad_comic_ids = through_model.objects.exclude(
@@ -104,7 +131,7 @@ def _fix_comic_m2m_integrity_errors(apps, comic_model, m2m_model_name, field_nam
     count_m2m, _ = m2m_rels_with_bad_m2m_ids.delete()
     count = count_comic + count_m2m
     if count:
-        LOG.verbose(f"Deleted {count} orphan relations to {field_name}")
+        LOG.info(f"Deleted {count} orphan relations to {field_name}")
     return bad_comics
 
 
@@ -155,7 +182,7 @@ def _find_fk_integrity_errors_with_models(
 
     count = invalid_host_objs.count()
     if count and log:
-        LOG.verbose(f"Found {host_model.__name__}s with bad {fk_field_name}: {count}")
+        LOG.info(f"Found {host_model.__name__}s with bad {fk_field_name}: {count}")
     return invalid_host_objs
 
 
@@ -176,6 +203,17 @@ def _delete_query(query, host_model_name, fk_model_name):
 def _delete_fk_integrity_errors(apps, host_model_name, fk_model_name, fk_field_name):
     """Delete objects with bad integrity."""
     try:
+        if not has_applied_migration(MIGRATION_0023) and (
+            host_model_name == "Creator"
+            or fk_model_name == "Creator"
+            or fk_field_name == "creators"
+        ):
+            LOG.debug(
+                "Skipping Creator integrity check until"
+                f"migration {MIGRATION_0023} applied."
+            )
+            return
+
         bad_host_objs = _find_fk_integrity_errors(
             apps, host_model_name, fk_model_name, fk_field_name
         )
@@ -202,39 +240,22 @@ def _null_missing_fk(host_model, fk_model, fk_field_name):
     if count:
         update_dict = {fk_field_name: None, "updated_at": Now()}
         query_missing_fks.update(**update_dict)
-        LOG.verbose(
-            f"Fixed {count} {host_model.__name__}s with missing {fk_field_name}"
-        )
+        LOG.info(f"Fixed {count} {host_model.__name__}s with missing {fk_field_name}")
 
 
-def _delete_userbookmark_integrity_errors(apps):
-    """Fix UserBookmarks with non codex model fields."""
-    # UserBookmarks that don't reference a valid comic are useless.
-    _delete_fk_integrity_errors(apps, "UserBookmark", "Comic", "comic")
-
-    # UserBookmarks that aren't linked to a valid session or a user are orphans
-    ubm_model = apps.get_model("codex", "UserBookmark")
-    _null_missing_fk(ubm_model, Session, "session")
-    _null_missing_fk(ubm_model, get_user_model(), "user")
-    orphan_ubms = ubm_model.objects.filter(session=None, user=None)
-    _delete_query(orphan_ubms, "UserBookmark", "session or user")
-
-
-def _delete_search_result_fk_errors(apps):
-    """Fix SearcResults with non valid fields."""
-    if not has_applied_migration(MIGRATION_0010):
+def _delete_bookmark_integrity_errors(apps):
+    """Fix Bookmarks with non codex model fields."""
+    if not has_applied_migration(MIGRATION_0018):
         return
-    comic_model = apps.get_model("codex", "comic")
-    search_query_model = apps.get_model("codex", "searchquery")
-    valid_comics = comic_model.objects.all()
-    valid_queries = search_query_model.objects.all()
-    search_result_model = apps.get_model("codex", "searchresult")
-    orphan_srs = search_result_model.objects.exclude(
-        query__in=valid_queries, comic__in=valid_comics
-    )
-    count, _ = orphan_srs.delete()
-    if count:
-        LOG.verbose(f"Deleted {count} orphan SearchResults")
+    # Bookmarks that don't reference a valid comic are useless.
+    _delete_fk_integrity_errors(apps, "Bookmark", "Comic", "comic")
+
+    # Bookmarks that aren't linked to a valid session or a user are orphans
+    bm_model = apps.get_model("codex", "Bookmark")
+    _null_missing_fk(bm_model, Session, "session")
+    _null_missing_fk(bm_model, get_user_model(), "user")
+    orphan_bms = bm_model.objects.filter(session=None, user=None)
+    _delete_query(orphan_bms, "Bookmark", "session or user")
 
 
 def _repair_library_groups(apps):
@@ -248,10 +269,8 @@ def _repair_library_groups(apps):
     bad_relations.delete()
 
 
-def _fix_db_integrity():
-    """Fix most of the Codex model integrity errors we can."""
-    LOG.verbose("Reparing database integrity...")
-    # DELETE things we can't fix
+def _delete_errors():
+    """DELETE things we can't fix."""
     for host_model_name in HAVE_LIBRARY_FKS:
         _delete_fk_integrity_errors(apps, host_model_name, "Library", "library")
 
@@ -265,20 +284,36 @@ def _fix_db_integrity():
                 apps, host_model_name, group_model_name, group_field_name
             )
 
-    for fk_model_name, fk_field_name in CREDIT_FIELDS.items():
-        _delete_fk_integrity_errors(apps, "Credit", fk_model_name, fk_field_name)
+    for fk_model_name, fk_field_name in CREATOR_FIELDS.items():
+        _delete_fk_integrity_errors(apps, "Creator", fk_model_name, fk_field_name)
 
-    _delete_userbookmark_integrity_errors(apps)
+    _delete_bookmark_integrity_errors(apps)
 
-    _repair_old_comic_folder_fks()
+    _delete_old_comic_folder_fks()
 
-    # REPAIR the objects that are left
+
+def _check_field_for_migration(model_name):
+    """Skip some models before migrations."""
+    for check_model_name, migration in SKIP_M2M_CHECKS.items():
+        if model_name == check_model_name and not has_applied_migration(migration):
+            LOG.debug(
+                f"Skipping {check_model_name} integrity check until"
+                f"migration {migration} applied."
+            )
+            return True
+    return False
+
+
+def _repair_integrity():
+    """REPAIR the objects that are left."""
     comic_model = apps.get_model("codex", "Comic")
     bad_comic_ids = comic_model.objects.none()
-    for m2m2_model_name, field_name in M2M_NAMES.items():
+    for m2m_model_name, field_name in M2M_NAMES.items():
+        if _check_field_for_migration(m2m_model_name):
+            continue
         try:
             bad_comic_ids |= _fix_comic_m2m_integrity_errors(
-                apps, comic_model, m2m2_model_name, field_name
+                apps, comic_model, m2m_model_name, field_name
             )
         except OperationalError as exc:
             ok = False
@@ -286,7 +321,7 @@ def _fix_db_integrity():
                 if arg in OPERATIONAL_ERRORS_BEFORE_MIGRATIONS:
                     ok = True
             if ok:
-                LOG.verbose(
+                LOG.info(
                     "Couldn't look for comics with folder integrity problems before "
                     "migration 0007. We'll get them on the next restart."
                 )
@@ -294,35 +329,32 @@ def _fix_db_integrity():
                 LOG.exception(exc)
     _mark_comics_with_bad_m2m_rels_for_update(comic_model, bad_comic_ids)
 
-    _delete_search_result_fk_errors(apps)
-
     _repair_library_groups(apps)
 
-    LOG.verbose("Done with database integrity check.")
-
 
 def repair_db():
     """Fix the db but trap errors if it goes wrong."""
     try:
-        if os.environ.get("CODEX_SKIP_INTEGRITY_CHECK"):
+        if SKIP_INTEGRITY_CHECK:
             LOG.info("Skipping integrity check")
             return
 
         if not has_applied_migration(MIGRATION_0005):
-            raise OperationalError(NO_0005_ARG)
-        _fix_db_integrity()
+            LOG.info(
+                "Not running integrity checks until migration"
+                f"{MIGRATION_0005} has been applied."
+            )
+            return
+
+        LOG.debug("Reparing database integrity...")
+        _delete_errors()
+        _repair_integrity()
+        LOG.info("Database integrity confirmed.")
     except OperationalError as exc:
-        ok = False
-        for arg in exc.args:
-            if arg in OK_EXC_ARGS:
-                ok = True
-                LOG.verbose(
-                    f"Not running integrity checks until migration {MIGRATION_0005}"
-                    " has been applied."
-                )
-                break
-        if not ok:
-            LOG.exception(exc)
+        if NO_MIGRATIONS_TABLE_ERROR in exc.args:
+            LOG.debug("Database not constructed yet, skipping integrity check.")
+        else:
+            raise
     except Exception as exc:
         LOG.exception(exc)
 
