@@ -1,27 +1,25 @@
 """Janitor Tasks."""
-from abc import ABC
 from dataclasses import dataclass
 
 
 @dataclass
-class JanitorTask(ABC):
+class JanitorTask:
     """Tasks for the janitor."""
 
-    pass
-
 
 @dataclass
 class JanitorBackupTask(JanitorTask):
     """Backup the database."""
 
-    pass
+
+@dataclass
+class JanitorRestartTask(JanitorTask):
+    """for restart."""
 
 
 @dataclass
-class JanitorRestartTask:
-    """for restart."""
-
-    pass
+class JanitorShutdownTask(JanitorTask):
+    """for shutdown."""
 
 
 @dataclass
@@ -35,25 +33,27 @@ class JanitorUpdateTask(JanitorTask):
 class JanitorVacuumTask(JanitorTask):
     """Vacuum the database."""
 
-    pass
-
-
-@dataclass
-class JanitorCleanSearchTask(JanitorTask):
-    """Clean the search db."""
-
-    pass
-
 
 @dataclass
 class JanitorCleanFKsTask(JanitorTask):
     """Clean unused foreign keys."""
 
-    pass
-
 
 @dataclass
 class JanitorClearStatusTask(JanitorTask):
     """Clear all librarian statuses."""
 
-    pass
+
+@dataclass
+class JanitorCleanupSessionsTask(JanitorTask):
+    """Cleanup Session table."""
+
+
+@dataclass
+class ForceUpdateAllFailedImportsTask(JanitorTask):
+    """Force update for failed imports in every library."""
+
+
+@dataclass
+class JanitorNightlyTask(JanitorTask):
+    """Submit all janitor nightly tasks to the queue."""
