@@ -1,50 +1,52 @@
 """Codex Django Models."""
+import base64
 import calendar
 import datetime
-import os
-
+import math
+import re
+import uuid
+from decimal import Decimal
 from pathlib import Path
 
 from django.conf import settings
 from django.contrib.auth.models import Group
 from django.contrib.sessions.models import Session
 from django.core.exceptions import ValidationError
-from django.db import connection
 from django.db.models import (
     CASCADE,
     BooleanField,
     CharField,
+    Choices,
     DateField,
     DateTimeField,
     DecimalField,
     DurationField,
     ForeignKey,
-    Index,
     JSONField,
     ManyToManyField,
     Model,
+    OneToOneField,
     PositiveIntegerField,
     PositiveSmallIntegerField,
+    TextChoices,
     TextField,
     URLField,
 )
-from django.db.models.enums import TextChoices
 from django.utils.translation import gettext_lazy as _
 
 from codex.librarian.covers.status import CoverStatusTypes
-from codex.librarian.db.status import ImportStatusTypes
+from codex.librarian.importer.status import ImportStatusTypes
 from codex.librarian.janitor.status import JanitorStatusTypes
 from codex.librarian.search.status import SearchIndexStatusTypes
 from codex.librarian.watchdog.status import WatchdogStatusTypes
-from codex.serializers.choices import CHOICES
-from codex.settings.logging import get_logger
-
+from codex.logger.logging import get_logger
 
 LOG = get_logger(__name__)
 
 
-MAX_PATH_LENGTH = 4095
-MAX_NAME_LENGTH = 128
+MAX_PATH_LEN = 4095
+MAX_NAME_LEN = 128
+MAX_FIELD_LEN = 32
 
 
 class BaseModel(Model):
@@ -66,7 +68,7 @@ class BrowserGroupModel(BaseModel):
     DEFAULT_NAME = ""
     ORDERING = ("name", "pk")
 
-    name = CharField(db_index=True, max_length=MAX_NAME_LENGTH, default=DEFAULT_NAME)
+    name = CharField(db_index=True, max_length=MAX_NAME_LEN, default=DEFAULT_NAME)
 
     class Meta:
         """Without this a real table is created and joined to."""
@@ -114,6 +116,7 @@ class Volume(BrowserGroupModel):
     """The volume of the series the comic belongs to."""
 
     ORDERING = ("series__name", "name", "pk")
+    YEAR_LEN = 4
 
     publisher = ForeignKey(Publisher, on_delete=CASCADE)
     imprint = ForeignKey(Imprint, on_delete=CASCADE)
@@ -125,6 +128,21 @@ class Volume(BrowserGroupModel):
 
         unique_together = ("name", "series")
 
+    @classmethod
+    def to_str(cls, name):
+        """Represent volume as a string."""
+        if not name:
+            vol = ""
+        elif len(name) == cls.YEAR_LEN:
+            vol = f"({name})"
+        else:
+            vol = "v" + name
+        return vol
+
+    def __str__(self):
+        """Represent volume as a string."""
+        return self.to_str(self.name)
+
 
 def validate_dir_exists(path):
     """Validate that a library exists."""
@@ -141,7 +159,7 @@ class Library(BaseModel):
     path = CharField(
         unique=True,
         db_index=True,
-        max_length=MAX_PATH_LENGTH,
+        max_length=MAX_PATH_LEN,
         validators=[validate_dir_exists],
     )
     events = BooleanField(db_index=True, default=True)
@@ -158,13 +176,13 @@ class Library(BaseModel):
     class Meta:
         """Pluralize."""
 
-        verbose_name_plural = "libraries"
+        verbose_name_plural = "Libraries"
 
 
 class NamedModel(BaseModel):
     """A for simple named tables."""
 
-    name = CharField(db_index=True, max_length=MAX_NAME_LENGTH)
+    name = CharField(db_index=True, max_length=MAX_NAME_LEN)
 
     class Meta:
         """Defaults to uniquely named, must be overridden."""
@@ -180,13 +198,21 @@ class NamedModel(BaseModel):
 class SeriesGroup(NamedModel):
     """A series group the series is part of."""
 
-    pass
 
-
-class StoryArc(NamedModel):
+class StoryArc(NamedModel, BrowserGroupModel):
     """A story arc the comic is part of."""
 
-    pass
+
+class StoryArcNumber(BaseModel):
+    """A story arc number the comic represents."""
+
+    story_arc = ForeignKey(StoryArc, db_index=True, on_delete=CASCADE)
+    number = PositiveIntegerField(null=True, default=None)
+
+    class Meta:
+        """Declare constraints and indexes."""
+
+        unique_together = ("story_arc", "number")
 
 
 class Location(NamedModel):
@@ -196,44 +222,32 @@ class Location(NamedModel):
 class Character(NamedModel):
     """A character that appears in the comic."""
 
-    pass
-
 
 class Team(NamedModel):
     """A team that appears in the comic."""
 
-    pass
-
 
 class Tag(NamedModel):
     """Arbitrary Metadata Tag."""
 
-    pass
-
 
 class Genre(NamedModel):
     """The genre the comic belongs to."""
 
-    pass
 
-
-class CreditPerson(NamedModel):
+class CreatorPerson(NamedModel):
     """Credited persons."""
 
-    pass
 
-
-class CreditRole(NamedModel):
+class CreatorRole(NamedModel):
     """A role for the credited person. Writer, Inker, etc."""
 
-    pass
 
-
-class Credit(BaseModel):
+class Creator(BaseModel):
     """A creator credit."""
 
-    person = ForeignKey(CreditPerson, on_delete=CASCADE)
-    role = ForeignKey(CreditRole, on_delete=CASCADE, null=True)
+    person = ForeignKey(CreatorPerson, on_delete=CASCADE)
+    role = ForeignKey(CreatorRole, on_delete=CASCADE, null=True)
 
     class Meta:
         """Constraints."""
@@ -245,25 +259,29 @@ class WatchedPath(BrowserGroupModel):
     """A filesystem path with data for Watchdog."""
 
     library = ForeignKey(Library, on_delete=CASCADE, db_index=True)
-    path = CharField(max_length=MAX_PATH_LENGTH, db_index=True)
+    path = CharField(max_length=MAX_PATH_LEN, db_index=True)
     stat = JSONField(null=True)
     parent_folder = ForeignKey(
         "Folder",
         on_delete=CASCADE,
         null=True,
     )
-    ZERO_STAT = [0, 0, 0, 0, 0, 0, 0, 0, 0.0, 0]
+    ZERO_STAT = (0, 0, 0, 0, 0, 0, 0, 0, 0.0, 0)
 
     def set_stat(self):
         """Set select stat params from the filesystem."""
-        st_record = os.stat(self.path)
+        st_record = Path(self.path).stat()
         # Converting os.stat directly to a list or tuple saves
         # mtime as an int and causes problems.
-        st = self.ZERO_STAT.copy()
+        st = list(self.ZERO_STAT)
         st[0] = st_record.st_mode
         st[1] = st_record.st_ino
-        # st_dev changes every time with docker
+        # st[2] = st_record.st_dev is ignored by diff
+        # st[3] = st_record.st_nlink
+        # st[4] = st_record.st_uid
+        # st[5] = st_record.st_gid
         st[6] = st_record.st_size
+        # st[7] = st_record.st_atime
         st[8] = st_record.st_mtime
         self.stat = st
 
@@ -282,23 +300,19 @@ class Folder(WatchedPath):
     """File system folder."""
 
 
-def validate_file_format_choice(choice):
-    """Validate file format."""
-    values = Comic.FileFormat.values
-    if choice not in values:
-        raise ValidationError(_(f"{choice} is not one of {values}"))
-
-
 class Comic(WatchedPath):
     """Comic metadata."""
 
-    class FileFormat(TextChoices):
+    class FileType(Choices):
         """Identifiers for file formats."""
 
-        COMIC = "comic"
-        PDF = "pdf"
+        CBZ = "CBZ"
+        CBR = "CBR"
+        CBT = "CBT"
+        PDF = "PDF"
 
     ORDERING = ("series__name", "volume__name", "issue", "issue_suffix", "name", "pk")
+    _RE_COMBINE_WHITESPACE = re.compile(r"\s+")
 
     # From BaseModel, but Comics are sorted by these so index them
     created_at = DateTimeField(auto_now_add=True, db_index=True)
@@ -321,9 +335,9 @@ class Comic(WatchedPath):
     month = PositiveSmallIntegerField(null=True)
     day = PositiveSmallIntegerField(null=True)
     # Summary
-    comments = TextField(null=True)
-    notes = TextField(null=True)
-    summary = TextField(null=True)
+    comments = TextField(default="")
+    notes = TextField(default="")
+    summary = TextField(default="")
     # Ratings
     community_rating = DecimalField(
         db_index=True, decimal_places=2, max_digits=5, default=None, null=True
@@ -331,23 +345,25 @@ class Comic(WatchedPath):
     critical_rating = DecimalField(
         db_index=True, decimal_places=2, max_digits=5, default=None, null=True
     )
-    age_rating = CharField(db_index=True, max_length=32, null=True)
+    age_rating = CharField(db_index=True, max_length=MAX_FIELD_LEN, default="")
     # alpha2 fields for countries
-    country = CharField(db_index=True, max_length=32, null=True)
-    language = CharField(db_index=True, max_length=32, null=True)
+    country = CharField(db_index=True, max_length=MAX_FIELD_LEN, default="")
+    language = CharField(db_index=True, max_length=MAX_FIELD_LEN, default="")
     # misc
-    format = CharField(db_index=True, max_length=32, null=True)
+    original_format = CharField(db_index=True, max_length=MAX_FIELD_LEN, default="")
     page_count = PositiveSmallIntegerField(db_index=True, default=0)
     read_ltr = BooleanField(db_index=True, default=True)
-    scan_info = CharField(max_length=MAX_NAME_LENGTH, null=True)
-    web = URLField(null=True)
+    scan_info = CharField(max_length=MAX_NAME_LEN, default="")
+    web = URLField(default="")
+    gtin = CharField(db_index=True, max_length=MAX_FIELD_LEN, default="")
+
     # ManyToMany
     characters = ManyToManyField(Character)
-    credits = ManyToManyField(Credit)
+    creators = ManyToManyField(Creator)
     genres = ManyToManyField(Genre)
     locations = ManyToManyField(Location)
     series_groups = ManyToManyField(SeriesGroup)
-    story_arcs = ManyToManyField(StoryArc)
+    story_arc_numbers = ManyToManyField(StoryArcNumber)
     tags = ManyToManyField(Tag)
     teams = ManyToManyField(Team)
     # Ignore these, they seem useless:
@@ -370,10 +386,11 @@ class Comic(WatchedPath):
     folders = ManyToManyField(Folder)
     max_page = PositiveSmallIntegerField(default=0)
     size = PositiveIntegerField(db_index=True)
-    file_format = CharField(
-        choices=FileFormat.choices,
-        max_length=max((len(val) for val in FileFormat.values)),
-        default=FileFormat.COMIC.value,
+    file_type = CharField(
+        choices=FileType.choices,
+        max_length=3,
+        blank=True,
+        default="",
     )
 
     class Meta:
@@ -388,10 +405,7 @@ class Comic(WatchedPath):
             year = datetime.MINYEAR
         else:
             year = min(max(self.year, datetime.MINYEAR), datetime.MAXYEAR)
-        if self.month is None:
-            month = 1
-        else:
-            month = min(max(self.month, 1), 12)
+        month = 1 if self.month is None else min(max(self.month, 1), 12)
 
         if self.day is None:
             day = 1
@@ -420,46 +434,91 @@ class Comic(WatchedPath):
         self.presave()
         super().save(*args, **kwargs)
 
+    @staticmethod
+    def _compute_zero_pad(issue_max):
+        """Compute zero padding for issues."""
+        if issue_max is None:
+            issue_max = 100
+        if issue_max < 1:
+            return 1
+        return math.floor(math.log10(issue_max)) + 1
+
+    @classmethod
+    def get_title(cls, obj, volume=True, issue_max=None, name=True):
+        """Create the comic title for display."""
+        names = []
+
+        # Series
+        if sn := obj.series_name:
+            names.append(sn)
+
+        # Volume
+        if volume and (vn := obj.volume_name):
+            vn = Volume.to_str(vn)
+            names.append(vn)
+
+        # Issue
+        issue = obj.issue.normalize() if obj.issue else Decimal(0)
+        zero_pad = cls._compute_zero_pad(issue_max)
+        if issue % 1 == 0:
+            precision = 0
+        else:
+            precision = 1
+            zero_pad += 2
+        issue_str = f"#{issue:0{zero_pad}.{precision}f}"
+        if issue_suffix := obj.issue_suffix:
+            issue_str += issue_suffix
+        names.append(issue_str)
+
+        # Title
+        if name and obj.name:
+            names.append(obj.name)
+
+        title = " ".join(filter(None, names)).strip(" .")
+        return cls._RE_COMBINE_WHITESPACE.sub(" ", title).strip()
+
+    @classmethod
+    def get_filename(cls, obj):
+        """Get the fileaname from dict."""
+        path = Path(obj.path)
+        return path.stem + path.suffix
+
+    def filename(self):
+        """Create a filename for download."""
+        return self.get_filename(self)
+
     def __str__(self):
         """Most common text representation for logging."""
-        names = []
-        if self.series.name:
-            names.append(self.series.name)
-        if self.volume.name:
-            names.append(self.volume.name)
-        if self.issue is not None:
-            names.append(f"#{self.issue:06.1f}")
-        if self.issue_suffix:
-            names.append(self.issue_suffix)
-        if self.name:
-            names.append(self.name)
-        return " ".join(names).strip()
+        return self.get_title(self)
 
 
-class AdminFlag(NamedModel):
+class AdminFlag(BaseModel):
     """Flags set by administrators."""
 
-    # TODO: SPA admin maybe change this to proper TextChoices.
-    ENABLE_FOLDER_VIEW = "Enable Folder View"
-    ENABLE_REGISTRATION = "Enable Registration"
-    ENABLE_NON_USERS = "Enable Non Users"
-    ENABLE_AUTO_UPDATE = "Enable Auto Update"
-    FLAG_NAMES = (
-        ENABLE_FOLDER_VIEW,
-        ENABLE_REGISTRATION,
-        ENABLE_NON_USERS,
-        ENABLE_AUTO_UPDATE,
-    )
-    DEFAULT_FALSE = (ENABLE_AUTO_UPDATE,)
+    class FlagChoices(Choices):
+        """Choices for Admin Flags."""
 
+        FOLDER_VIEW = "FV"
+        REGISTRATION = "RG"
+        NON_USERS = "NU"
+        AUTO_UPDATE = "AU"
+        SEARCH_INDEX_OPTIMIZE = "SO"
+
+    FALSE_DEFAULTS = frozenset((FlagChoices.AUTO_UPDATE,))
+
+    key = CharField(db_index=True, max_length=2, choices=FlagChoices.choices)
     on = BooleanField(default=True)
 
+    class Meta:
+        """Constraints."""
+
+        unique_together = ("key",)
+
 
 def cascade_if_user_null(collector, field, sub_objs, _using):
-    """
-    Cascade only if the user field is null.
+    """Cascade only if the user field is null.
 
-    Do this to keep deleting ephemeral session data from UserBookmark table.
+    Do this to keep deleting ephemeral session data from Bookmark table.
     Adapted from:
     https://github.com/django/django/blob/master/django/db/models/deletion.py#L23
     """
@@ -483,17 +542,17 @@ def cascade_if_user_null(collector, field, sub_objs, _using):
         collector.add_field_update(field, None, sub_objs)
 
 
-def validate_fit_to_choice(choice):
-    """Validate fit to choice."""
-    # Choices is loaded after migration time and after definition time.
-    values = CHOICES["fitTo"]
-    if choice is not None and choice not in values:
-        raise ValidationError(_(f"{choice} is not one of {values}"))
-
-
-class UserBookmark(BaseModel):
+class Bookmark(BaseModel):
     """Persist user's bookmarks and settings."""
 
+    class FitTo(Choices):
+        """Identifiers for Readder fit_to choices."""
+
+        SCREEN = "S"
+        WIDTH = "W"
+        HEIGHT = "H"
+        ORIG = "O"
+
     user = ForeignKey(
         settings.AUTH_USER_MODEL, db_index=True, on_delete=CASCADE, null=True
     )
@@ -501,15 +560,17 @@ class UserBookmark(BaseModel):
         Session, db_index=True, on_delete=cascade_if_user_null, null=True
     )
     comic = ForeignKey(Comic, db_index=True, on_delete=CASCADE)
-    bookmark = PositiveSmallIntegerField(db_index=True, null=True)
+    page = PositiveSmallIntegerField(db_index=True, null=True)
     finished = BooleanField(default=False, db_index=True)
     fit_to = CharField(
-        validators=[validate_fit_to_choice],
-        null=True,
-        default=None,
-        max_length=len("SCREEN"),
+        choices=FitTo.choices,
+        blank=True,
+        default="",
+        max_length=1,
     )
     two_pages = BooleanField(default=None, null=True)
+    read_in_reverse = BooleanField(default=None, null=True)
+    vertical = BooleanField(default=None, null=True)
 
     class Meta:
         """Constraints."""
@@ -523,10 +584,10 @@ class FailedImport(WatchedPath):
     def set_reason(self, exc):
         """Can't do this in save() because it breaks update_or_create."""
         reason = str(exc)
-        suffixes = (f": {self.path}", f": '{self.path}'")
+        suffixes = (f": {self.path}", f": {self.path!r}")
         for suffix in suffixes:
             reason = reason.removesuffix(suffix)
-        reason = reason[:MAX_NAME_LENGTH]
+        reason = reason[:MAX_NAME_LEN]
         self.name = reason.strip()
 
     class Meta:
@@ -535,85 +596,69 @@ class FailedImport(WatchedPath):
         unique_together = ("library", "path")
 
 
-class SearchQuery(Model):
-    """Search queries."""
-
-    QUERY_MAX_LENGTH = 256
-
-    text = CharField(db_index=True, unique=True, max_length=QUERY_MAX_LENGTH)
-    used_at = DateTimeField(auto_now_add=True, db_index=True)
-
-
-class SearchResult(Model):
-    """results model."""
-
-    query = ForeignKey(SearchQuery, on_delete=CASCADE)
-    comic = ForeignKey(Comic, on_delete=CASCADE)
-    score = PositiveSmallIntegerField()
-
-    @classmethod
-    def truncate_and_reset(cls):
-        """Nuke this table and reset the autoincrementer."""
-        cls.objects.all().delete()
-        with connection.cursor() as cursor:
-            cursor.execute(
-                "UPDATE sqlite_sequence SET seq=1 WHERE name=%s", [cls._meta.db_table]
-            )
-
-    class Meta:
-        """These are usually looked up by comic & autoquery hash."""
-
-        unique_together = ("query", "comic")
-        indexes = [Index(fields=["comic", "query"])]
-
-
-class LibrarianStatus(NamedModel):
+class LibrarianStatus(BaseModel):
     """Active Library Tasks."""
 
-    DEFAULT_PARAMS = {
-        "name": "",
-        "preactive": False,
-        "complete": 0,
-        "total": 0,
-        "active": None,
-    }
-    TYPES = (
-        *CoverStatusTypes.values(),
-        *ImportStatusTypes.values(),
-        *JanitorStatusTypes.values(),
-        *SearchIndexStatusTypes.values(),
-        *WatchdogStatusTypes.values(),
+    CHOICES = (
+        CoverStatusTypes.choices
+        + ImportStatusTypes.choices
+        + JanitorStatusTypes.choices
+        + SearchIndexStatusTypes.choices
+        + WatchdogStatusTypes.choices
     )
-    type = CharField(db_index=True, max_length=32)
+
+    status_type = CharField(db_index=True, max_length=3, choices=CHOICES)
     preactive = BooleanField(default=False)
-    complete = PositiveSmallIntegerField(default=0)
-    total = PositiveSmallIntegerField(default=0)
+    complete = PositiveSmallIntegerField(null=True, default=None)
+    total = PositiveSmallIntegerField(null=True, default=None)
     active = DateTimeField(null=True, default=None)
+    subtitle = CharField(db_index=True, max_length=MAX_NAME_LEN)
 
     class Meta:
         """Constraints."""
 
-        unique_together = ("type", "name")
+        unique_together = ("status_type", "subtitle")
+        verbose_name_plural = "LibrarianStatuses"
 
 
-class Timestamp(NamedModel):
-    """Timestamp."""
+class Timestamp(BaseModel):
+    """Timestamped Named Strings."""
 
-    COVERS = "covers"
-    JANITOR = "janitor"
-    SEARCH_INDEX = "search_index"
-    CODEX_VERSION = "codex_version"
-    XAPIAN_INDEX_UUID = "xapian_index_uuid"
-    NAMES = (COVERS, JANITOR, SEARCH_INDEX, CODEX_VERSION, XAPIAN_INDEX_UUID)
+    class TimestampChoices(TextChoices):
+        """Choices for Timestamps."""
 
-    version = CharField(max_length=32, null=True, default=None)
+        COVERS = "CV", _("Covers")
+        JANITOR = "JA", _("Janitor")
+        CODEX_VERSION = "VR", _("Codex Version")
+        SEARCH_INDEX_UUID = "SI", _("Search Index UUID")
+        API_KEY = "AP", _("API Key")
+
+    key = CharField(db_index=True, max_length=2, choices=TimestampChoices.choices)
+    version = CharField(max_length=MAX_FIELD_LEN, default="")
 
     @classmethod
-    def touch(cls, name):
+    def touch(cls, choice):
         """Touch a timestamp."""
-        cls.objects.get(name=name).save()
+        cls.objects.get(key=choice.value).save()
 
-    @classmethod
-    def get(cls, name):
-        """Get the timestamp."""
-        return cls.objects.get(name=name).updated_at.timestamp()
+    def save_uuid_version(self):
+        """Create base64 uuid."""
+        uuid_bytes = uuid.uuid4().bytes
+        b64_bytes = base64.urlsafe_b64encode(uuid_bytes)
+        self.version = b64_bytes.decode("utf-8").replace("=", "")
+        self.save()
+
+    class Meta:
+        """Constraints."""
+
+        unique_together = ("key",)
+
+    def __str__(self):
+        """Print name for choice."""
+        return self.TimestampChoices(self.key).name
+
+
+class UserActive(BaseModel):
+    """User last active record."""
+
+    user = OneToOneField(settings.AUTH_USER_MODEL, db_index=True, on_delete=CASCADE)
