@@ -1,4 +1,5 @@
 """Codex Auth Serializers."""
+from types import MappingProxyType
 
 from django.contrib.auth.models import User
 from rest_framework.fields import BooleanField, CharField
@@ -14,18 +15,35 @@ from codex.models import AdminFlag
 class UserSerializer(ModelSerializer):
     """Serialize User model for UI."""
 
-    enableNonUsers = SerializerMethodField()  # noqa: N815
+    _ADMIN_FLAG_KEYS = (
+        AdminFlag.FlagChoices.NON_USERS.value,
+        AdminFlag.FlagChoices.REGISTRATION.value,
+    )
 
-    def get_enableNonUsers(self, obj):  # noqa: N802
-        """Piggyback on user objects. A little awkward."""
-        enu_flag = AdminFlag.objects.only("on").get(name=AdminFlag.ENABLE_NON_USERS)
-        return enu_flag.on
+    admin_flags = SerializerMethodField()
+
+    def get_admin_flags(self, *args):
+        """Piggyback admin flags on the user object."""
+        flags = AdminFlag.objects.filter(key__in=self._ADMIN_FLAG_KEYS).values(
+            "name", "on"
+        )
+        admin_flags = {}
+        for flag in flags:
+            name = flag["name"]
+            key = name[0].lower() + name[1:].replace(" ", "")
+            admin_flags[key] = flag["on"]
+        return admin_flags
 
     class Meta:
         """Model spec."""
 
         model = User
-        fields = ("pk", "username", "is_staff", "enableNonUsers")
+        fields = (
+            "pk",
+            "username",
+            "is_staff",
+            "admin_flags",
+        )
         read_only_fields = fields
 
 
@@ -43,7 +61,7 @@ class UserCreateSerializer(ModelSerializer, TimezoneSerializer):
 
         model = User
         fields = ("username", "password", "timezone")
-        extra_kwargs = {"password": {"write_only": True}}
+        extra_kwargs = MappingProxyType({"password": {"write_only": True}})
 
 
 class UserLoginSerializer(UserCreateSerializer):
@@ -56,7 +74,8 @@ class UserLoginSerializer(UserCreateSerializer):
         """Explicit meta inheritance required."""
 
 
-class RegistrationEnabledSerializer(Serializer):
-    """Serialize one admin flag."""
+class AuthAdminFlagsSerializer(Serializer):
+    """Admin flags related to auth."""
 
-    enableRegistration = BooleanField(read_only=True)  # noqa: N815
+    non_users = BooleanField(read_only=True)
+    registration = BooleanField(read_only=True)
