@@ -1,5 +1,5 @@
 """Serializers for the browser view."""
-from abc import ABC
+from abc import ABC, abstractmethod
 
 from django.core.exceptions import ValidationError
 from django.utils.translation import gettext_lazy as _
@@ -8,15 +8,16 @@ from rest_framework.serializers import (
     CharField,
     ChoiceField,
     DecimalField,
-    Field,
+    FloatField,
     IntegerField,
     ListField,
     Serializer,
 )
 
 from codex.serializers.choices import CHOICES, VUETIFY_NULL_CODE
-from codex.serializers.mixins import UNIONFIX_PREFIX, BrowserCardOPDSBaseSerializer
-
+from codex.serializers.mixins import (
+    BrowserAggregateSerializerMixin,
+)
 
 VUETIFY_NULL_CODE_STR = str(VUETIFY_NULL_CODE)
 
@@ -37,8 +38,7 @@ def validate_decades(decades):
 
 
 def validate_int_null(values):
-    """
-    Use a special code for null.
+    """Use a special code for null.
 
     Because if a vuetify component has a null key it changes it to the
     array index.
@@ -50,8 +50,7 @@ def validate_int_null(values):
 
 
 def validate_str_null(values):
-    """
-    Use a special code for null.
+    """Use a special code for null.
 
     Because if a vuetify component has a null key it changes it to the
     array index. This is the version for CharFields.
@@ -74,8 +73,14 @@ class BrowserSettingsShowGroupFlagsSerializer(Serializer):
 class FilterListField(ListField, ABC):
     """Filter List field with custom arguments."""
 
-    CHILD_CLASS = Field
-    VALIDATORS = tuple()
+    @property
+    @classmethod
+    @abstractmethod
+    def CHILD_CLASS(cls):  # noqa: N802
+        """Child field class."""
+        raise NotImplementedError
+
+    VALIDATORS = ()
 
     def __init__(self, *args, **kwargs):
         """Apply the subclass's arguments."""
@@ -87,17 +92,17 @@ class FilterListField(ListField, ABC):
         )
 
 
-class DecimalListField(FilterListField):
+class FloatListField(FilterListField):
     """Decimal List Field with validation."""
 
-    CHILD_CLASS = DecimalField
+    CHILD_CLASS = FloatField  # type: ignore
     VALIDATORS = (validate_int_null,)
 
 
 class IntListField(FilterListField):
     """Integer List Field with validation."""
 
-    CHILD_CLASS = IntegerField
+    CHILD_CLASS = IntegerField  # type: ignore
     VALIDATORS = (validate_int_null,)
 
 
@@ -110,7 +115,7 @@ class DecadeListField(IntListField):
 class CharListField(FilterListField):
     """Char List Field with validation."""
 
-    CHILD_CLASS = CharField
+    CHILD_CLASS = CharField  # type: ignore
     VALIDATORS = (validate_str_null,)
 
 
@@ -121,69 +126,63 @@ class BrowserSettingsFilterSerializer(Serializer):
         choices=tuple(CHOICES["bookmarkFilter"].keys()), required=False
     )
     # Dynamic filters
-    community_rating = DecimalListField(
-        max_digits=5,
-        decimal_places=2,
-        coerce_to_string=False,
-    )
-    characters = IntListField()
-    country = CharListField()
-    creators = IntListField()
-    critical_rating = DecimalListField(
-        max_digits=5,
-        decimal_places=2,
-        coerce_to_string=False,
-    )
-    decade = DecadeListField()
-    format = CharListField()
-    genres = IntListField()
-    language = CharListField()
+    age_rating = CharListField(allow_blank=True, required=False)
+    characters = IntListField(required=False)
+    community_rating = FloatListField(required=False)
+    country = CharListField(allow_blank=True, required=False)
+    creators = IntListField(required=False)
+    critical_rating = FloatListField(required=False)
+    decade = DecadeListField(required=False)
+    file_type = CharListField(allow_blank=True, required=False)
+    genres = IntListField(required=False)
+    language = CharListField(allow_blank=True, required=False)
     locations = IntListField()
-    age_rating = CharListField()
-    read_ltr = ListField(child=BooleanField(), required=False)
-    series_groups = IntListField()
-    story_arcs = IntListField()
-    tags = IntListField()
-    teams = IntListField()
-    year = IntListField()
+    original_format = CharListField(required=False, allow_blank=True)
+    read_ltr = ListField(child=BooleanField(allow_null=True), required=False)
+    series_groups = IntListField(required=False)
+    story_arcs = IntListField(required=False)
+    tags = IntListField(required=False)
+    teams = IntListField(required=False)
+    year = IntListField(required=False)
 
 
 class BrowserSettingsSerializer(Serializer):
-    """
-    Browser Settings that the user can change.
+    """Browser Settings that the user can change.
 
     This is the only browse serializer that's submitted.
     It is also sent to the browser as part of BrowserOpenedSerializer.
     """
 
     filters = BrowserSettingsFilterSerializer(required=False)
-    autoquery = CharField(allow_blank=True, required=False)
-    top_group = ChoiceField(choices=tuple(CHOICES["topGroup"].keys()), required=False)
     order_by = ChoiceField(choices=tuple(CHOICES["orderBy"].keys()), required=False)
     order_reverse = BooleanField(required=False)
+    q = CharField(allow_blank=True, required=False)
+    query = CharField(allow_blank=True, required=False)  # OPDS 2.0
     show = BrowserSettingsShowGroupFlagsSerializer(required=False)
+    twenty_four_hour_time = BooleanField(required=False)
+    top_group = ChoiceField(choices=tuple(CHOICES["topGroup"].keys()), required=False)
+    opds_metadata = BooleanField(required=False)
+    limit = IntegerField(required=False)
 
 
-class BrowserCardSerializer(BrowserCardOPDSBaseSerializer):
+class BrowserCardSerializer(BrowserAggregateSerializerMixin):
     """Browse card displayed in the browser."""
 
-    publisher_name = CharField(
-        read_only=True, source=UNIONFIX_PREFIX + "publisher_name"
-    )
-    series_name = CharField(read_only=True, source=UNIONFIX_PREFIX + "series_name")
-    volume_name = CharField(read_only=True, source=UNIONFIX_PREFIX + "volume_name")
-    name = CharField(read_only=True, source=UNIONFIX_PREFIX + "name")
+    pk = IntegerField(read_only=True)
+    publisher_name = CharField(read_only=True)
+    series_name = CharField(read_only=True)
+    volume_name = CharField(read_only=True)
+    name = CharField(read_only=True)
     issue = DecimalField(
         max_digits=16,
         decimal_places=3,
         read_only=True,
         coerce_to_string=False,
-        source=UNIONFIX_PREFIX + "issue",
     )
-    issue_suffix = CharField(read_only=True, source=UNIONFIX_PREFIX + "issue_suffix")
-    order_value = CharField(read_only=True, source=UNIONFIX_PREFIX + "order_value")
-    page_count = IntegerField(read_only=True, source=UNIONFIX_PREFIX + "page_count")
-    read_ltr = BooleanField(read_only=True, source=UNIONFIX_PREFIX + "read_ltr")
+    issue_suffix = CharField(read_only=True)
+    order_value = CharField(read_only=True)
+    page_count = IntegerField(read_only=True)
+    read_ltr = BooleanField(read_only=True)
 
 
 class BrowserRouteSerializer(Serializer):
@@ -197,13 +196,13 @@ class BrowserRouteSerializer(Serializer):
 class BrowserAdminFlagsSerializer(Serializer):
     """These choices change with browse context."""
 
-    enable_folder_view = BooleanField(read_only=True)
+    folder_view = BooleanField(read_only=True)
 
 
 class BrowserTitleSerializer(Serializer):
     """Elements for constructing the browse title."""
 
-    parent_name = CharField(read_only=True, allow_null=True)
+    parent_name = CharField(read_only=True)
     group_name = CharField(read_only=True)
     group_count = IntegerField(read_only=True, allow_null=True)
 
@@ -211,39 +210,48 @@ class BrowserTitleSerializer(Serializer):
 class BrowserPageSerializer(Serializer):
     """The main browse list."""
 
-    NUM_AUTOCOMPLETE_QUERIES = 10
-
+    admin_flags = BrowserAdminFlagsSerializer(read_only=True)
     browser_title = BrowserTitleSerializer(read_only=True)
-    model_group = CharField(read_only=True)
-    up_route = BrowserRouteSerializer(allow_null=True, read_only=True)
-    obj_list = ListField(
-        child=BrowserCardSerializer(read_only=True), allow_empty=True, read_only=True
-    )
+    covers_timestamp = IntegerField(read_only=True)
     issue_max = DecimalField(
         max_digits=16,
         decimal_places=3,
         read_only=True,
         coerce_to_string=False,
     )
-    num_pages = IntegerField(read_only=True)
-    admin_flags = BrowserAdminFlagsSerializer(read_only=True)
     libraries_exist = BooleanField(read_only=True)
-    queries = ListField(
-        child=CharField(read_only=True), allow_empty=True, read_only=True
-    )
-    covers_timestamp = IntegerField(read_only=True)
+    model_group = CharField(read_only=True)
+    num_pages = IntegerField(read_only=True)
+    groups = BrowserCardSerializer(allow_empty=True, read_only=True, many=True)
+    books = BrowserCardSerializer(allow_empty=True, read_only=True, many=True)
+    up_route = BrowserRouteSerializer(allow_null=True, read_only=True)
 
 
-class VersionsSerializer(Serializer):
-    """Codex version information."""
+class BrowserChoicesSerializer(Serializer):
+    """Named Model Serailizer with pk = char hack for languages & countries."""
 
-    installed = CharField(read_only=True)
-    latest = CharField(read_only=True)
+    pk = CharField(read_only=True)
+    name = CharField(read_only=True)
 
 
-class BrowserOpenedSerializer(Serializer):
-    """Component open settings."""
+class BrowserFilterChoicesSerializer(Serializer):
+    """All dynamic filters."""
 
-    settings = BrowserSettingsSerializer(read_only=True)
-    browser_page = BrowserPageSerializer(read_only=True)
-    versions = VersionsSerializer(read_only=True)
+    age_rating = BooleanField(read_only=True)
+    community_rating = BooleanField(read_only=True)
+    characters = BooleanField(read_only=True)
+    country = BooleanField(read_only=True)
+    critical_rating = BooleanField(read_only=True)
+    creators = BooleanField(read_only=True)
+    decade = BooleanField(read_only=True)
+    genres = BooleanField(read_only=True)
+    file_type = BooleanField(read_only=True)
+    language = BooleanField(read_only=True)
+    locations = BooleanField(read_only=True)
+    original_format = BooleanField(read_only=True)
+    read_ltr = BooleanField(read_only=True)
+    series_groups = BooleanField(read_only=True)
+    story_arcs = BooleanField(read_only=True)
+    tags = BooleanField(read_only=True)
+    teams = BooleanField(read_only=True)
+    year = BooleanField(read_only=True)
