@@ -1,6 +1,7 @@
 """Serializers for codex models."""
 import pycountry
-
+from drf_spectacular.types import OpenApiTypes
+from drf_spectacular.utils import extend_schema_field
 from rest_framework.serializers import (
     IntegerField,
     ModelSerializer,
@@ -9,18 +10,21 @@ from rest_framework.serializers import (
 )
 
 from codex.models import (
+    Bookmark,
     Character,
     Comic,
-    Credit,
-    CreditPerson,
-    CreditRole,
+    Creator,
+    CreatorPerson,
+    CreatorRole,
     Genre,
     Imprint,
+    LibrarianStatus,
     Location,
     Publisher,
     Series,
     SeriesGroup,
     StoryArc,
+    StoryArcNumber,
     Tag,
     Team,
     Volume,
@@ -28,8 +32,7 @@ from codex.models import (
 
 
 class PyCountrySerializer(Serializer):
-    """
-    PyCountry tag serializer to include long names.
+    """PyCountry tag serializer to include long names.
 
     Takes a single string value and serializes the value to
     a pk attribute and a pycountry long name lookup of that value to
@@ -37,6 +40,7 @@ class PyCountrySerializer(Serializer):
     """
 
     LOOKUP_MODULE = pycountry.countries
+    _ALPHA_2_LEN = 2
 
     pk = SerializerMethodField()
     name = SerializerMethodField()
@@ -46,26 +50,29 @@ class PyCountrySerializer(Serializer):
 
         abstract = True
 
-    def get_pk(self, obj):
+    @extend_schema_field(OpenApiTypes.STR)
+    def get_pk(self, pk):
         """Return submitted value as the key."""
-        return obj
+        return pk
 
-    def get_name(self, obj):
+    @classmethod
+    def lookup_name(cls, lookup_module, name):
         """Lookup the name with pycountry, just copy the key on fail."""
-        if obj is None:
-            # This never seems to get called so I do it on the front end.
-            return "None"
-        if len(obj) == 2:
-            # fix for https://github.com/flyingcircusio/pycountry/issues/41
-            lookup_obj = self.LOOKUP_MODULE.get(alpha_2=obj)
-        else:
-            lookup_obj = self.LOOKUP_MODULE.lookup(obj)
-        if lookup_obj:
-            value = lookup_obj.name
-        else:
-            # If lookup fails, return the key as the name
-            value = obj
-        return value
+        if not name:
+            return ""
+        # fix for https://github.com/flyingcircusio/pycountry/issues/41
+        lookup_obj = (
+            lookup_module.get(alpha_2=name)
+            if len(name) == cls._ALPHA_2_LEN
+            else lookup_module.lookup(name)
+        )
+        # If lookup fails, return the key as the name
+        return lookup_obj.name if lookup_obj else name
+
+    @extend_schema_field(OpenApiTypes.STR)
+    def get_name(self, name):
+        """Lookup the name with pycountry, just copy the key on fail."""
+        return self.lookup_name(self.LOOKUP_MODULE, name)
 
 
 class LanguageSerializer(PyCountrySerializer):
@@ -77,8 +84,6 @@ class LanguageSerializer(PyCountrySerializer):
 class CountrySerializer(PyCountrySerializer):
     """Pycountry serializer for country field."""
 
-    pass
-
 
 class NamedModelMeta:
     """Meta class for named models."""
@@ -91,17 +96,11 @@ class NamedModelMeta:
 ##########
 
 
-class GroupModelMeta:
-    """Meta class for group models."""
-
-    fields = NamedModelMeta.fields
-
-
 class NamedModelSerializer(ModelSerializer):
     """A common class for NamedModels."""
 
-    class Meta:
-        """Abstract class."""
+    class Meta(NamedModelMeta):
+        """Not Abstract."""
 
         abstract = True
 
@@ -109,7 +108,7 @@ class NamedModelSerializer(ModelSerializer):
 class GroupModelSerializer(NamedModelSerializer):
     """A common class for BrowserGroupModels."""
 
-    class Meta:
+    class Meta(NamedModelMeta):
         """Abstract class."""
 
         abstract = True
@@ -118,7 +117,7 @@ class GroupModelSerializer(NamedModelSerializer):
 class PublisherSerializer(GroupModelSerializer):
     """Publisher Model."""
 
-    class Meta(GroupModelMeta):
+    class Meta(NamedModelMeta):
         """Configure model."""
 
         model = Publisher
@@ -127,7 +126,7 @@ class PublisherSerializer(GroupModelSerializer):
 class ImprintSerializer(GroupModelSerializer):
     """Imprint Model."""
 
-    class Meta(GroupModelMeta):
+    class Meta(NamedModelMeta):
         """Configure model."""
 
         model = Imprint
@@ -136,19 +135,21 @@ class ImprintSerializer(GroupModelSerializer):
 class SeriesSerializer(GroupModelSerializer):
     """Series Model."""
 
-    class Meta(GroupModelMeta):
+    class Meta(NamedModelMeta):
         """Configure model."""
 
         model = Series
+        # fields = (*NamedModelMeta.fields, "volume_count")
 
 
 class VolumeSerializer(GroupModelSerializer):
     """Volume Model."""
 
-    class Meta(GroupModelMeta):
+    class Meta(NamedModelMeta):
         """Configure model."""
 
         model = Volume
+        # fields = (*NamedModelMeta.fields, "issue_count")
 
 
 ##############
@@ -156,34 +157,34 @@ class VolumeSerializer(GroupModelSerializer):
 ##############
 
 
-class CreditPersonSerializer(NamedModelSerializer):
-    """CreditPerson model."""
+class CreatorPersonSerializer(NamedModelSerializer):
+    """CreatorPerson model."""
 
     class Meta(NamedModelMeta):
         """Configure model."""
 
-        model = CreditPerson
+        model = CreatorPerson
 
 
-class CreditRoleSerializer(NamedModelSerializer):
-    """CreditRole model."""
+class CreatorRoleSerializer(NamedModelSerializer):
+    """CreatorRole model."""
 
     class Meta(NamedModelMeta):
         """Configure model."""
 
-        model = CreditRole
+        model = CreatorRole
 
 
-class CreditSerializer(ModelSerializer):
-    """Credit model serializer."""
+class CreatorSerializer(ModelSerializer):
+    """Creator model serializer."""
 
-    role = CreditRoleSerializer()
-    person = CreditPersonSerializer()
+    role = CreatorRoleSerializer()
+    person = CreatorPersonSerializer()
 
     class Meta:
         """Model spec."""
 
-        model = Credit
+        model = Creator
         fields = ("pk", "person", "role")
         depth = 1
 
@@ -233,6 +234,20 @@ class StoryArcSerializer(NamedModelSerializer):
         model = StoryArc
 
 
+class StoryArcNumberSerializer(Serializer):
+    """StoryArc model."""
+
+    story_arc = StoryArcSerializer()
+    number = IntegerField(read_only=True, allow_null=True)
+
+    class Meta(NamedModelMeta):
+        """Configure model."""
+
+        model = StoryArcNumber
+        fields = ("pk", "story_arc", "number")
+        depth = 1
+
+
 class TagSerializer(NamedModelSerializer):
     """Tag model."""
 
@@ -254,21 +269,12 @@ class TeamSerializer(NamedModelSerializer):
 class ComicSerializer(ModelSerializer):
     """Serialize a comic object for the metadata dialog."""
 
-    def __init__(self, *args, **kwargs):
-        """Dynamically whitelist fields."""
-        fields = kwargs.pop("fields", None)
-        # Use the fields argument to remove fields not in the list.
-        if fields:
-            allowed_fields = set(fields)
-            existing_fields = set(self.fields.keys())
-            excluded_fields = existing_fields - allowed_fields
-            for field in excluded_fields:
-                self.fields.pop(field)
-        super().__init__(*args, **kwargs)
+    # Easier than specifying fields in Meta
+    pk = IntegerField(source="id")
 
     # Annotations
-    issue_count = IntegerField(allow_null=True)
-    volume_count = IntegerField(allow_null=True)
+    # issue_count = IntegerField(allow_null=True)
+    # volume_count = IntegerField(allow_null=True)
 
     # Special Serialization with pycountry
     country = CountrySerializer(allow_null=True)
@@ -284,15 +290,55 @@ class ComicSerializer(ModelSerializer):
     characters = CharacterSerializer(many=True, allow_null=True)
     genres = GenreSerializer(many=True, allow_null=True)
     locations = LocationSerializer(many=True, allow_null=True)
-    seriesGroups = SeriesGroupSerializer(many=True, allow_null=True)  # noqa: N815
-    storyArcs = StoryArcSerializer(many=True, allow_null=True)  # noqa: N815
+    series_groups = SeriesGroupSerializer(many=True, allow_null=True)
+    story_arc_numbers = StoryArcNumberSerializer(
+        many=True,
+        allow_null=True,
+    )
     tags = TagSerializer(many=True, allow_null=True)
     teams = TeamSerializer(many=True, allow_null=True)
-    credits = CreditSerializer(many=True, allow_null=True)
+    creators = CreatorSerializer(many=True, allow_null=True)
 
     class Meta:
         """Configure the model."""
 
         model = Comic
-        fields = "__all__"  # Overridden dynamically by constructor param
+        exclude = ("folders", "parent_folder", "stat")
         depth = 1
+
+
+class LibrarianStatusSerializer(ModelSerializer):
+    """Serializer Librarian task statuses."""
+
+    class Meta:
+        """Configure the model."""
+
+        model = LibrarianStatus
+        exclude = ("active", "created_at", "updated_at")
+
+
+class BookmarkSerializer(ModelSerializer):
+    """Serializer Bookmark."""
+
+    class Meta:
+        """Configure the model."""
+
+        model = Bookmark
+        fields = (
+            "finished",
+            "fit_to",
+            "page",
+            "two_pages",
+            "read_in_reverse",
+            "vertical",
+        )
+
+
+class BookmarkFinishedSerializer(ModelSerializer):
+    """The finished field of the Bookmark."""
+
+    class Meta:
+        """Model spec."""
+
+        model = Bookmark
+        fields = ("finished",)
