@@ -1,31 +1,29 @@
 """Abstract Thread worker for doing queued tasks."""
 import time
-
 from abc import ABC, abstractmethod
-from multiprocessing import Queue
-from queue import Empty
+from queue import Empty, SimpleQueue
 from threading import Thread
 
-from codex.settings.logging import get_logger
-
-
-LOG = get_logger(__name__)
+from codex.worker_base import WorkerBaseMixin
 
 
 class BreakLoopError(Exception):
     """Simple way to break out of function nested loop."""
 
-    pass
 
-
-class NamedThread(Thread, ABC):
+class NamedThread(Thread, WorkerBaseMixin, ABC):
     """A thread that sets its name for ps."""
 
-    NAME = "abstract-named-thread"
+    def __init__(self, *args, **kwargs):
+        """Initialize queues."""
+        librarian_queue = kwargs.pop("librarian_queue")
+        log_queue = kwargs.pop("log_queue")
+        self.init_worker(log_queue, librarian_queue)
+        super().__init__(*args, **kwargs)
 
     def run_start(self):
         """First thing to do when running a new thread."""
-        LOG.verbose(f"Started {self.NAME} thread")
+        self.log.debug(f"Started {self.__class__.__name__}")
 
 
 class QueuedThread(NamedThread, ABC):
@@ -34,25 +32,22 @@ class QueuedThread(NamedThread, ABC):
     SHUTDOWN_MSG = "shutdown"
     SHUTDOWN_TIMEOUT = 5
 
-    NAME = "abstract-queued-thread"
-
-    def __init__(self):
+    def __init__(self, *args, **kwargs):
         """Initialize with overridden name and as a daemon thread."""
-        self.queue = Queue()
-        super().__init__(name=self.NAME, daemon=True)
+        self.queue = kwargs.pop("queue", SimpleQueue())
+        super().__init__(*args, name=self.__class__.__name__, daemon=True, **kwargs)
 
     @abstractmethod
     def process_item(self, item):
         """Process one item from the queue."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def get_timeout(self):
         """Set no timeout by default."""
-        return None
+        return
 
     def timed_out(self):
         """Override to things on queue timeout."""
-        pass
 
     def _check_item(self):
         """Get items, with timeout. Check for shutdown and Empty."""
@@ -60,7 +55,7 @@ class QueuedThread(NamedThread, ABC):
         try:
             item = self.queue.get(timeout=timeout)
             if item == self.SHUTDOWN_MSG:
-                raise BreakLoopError()
+                raise BreakLoopError  # TRY301
             self.process_item(item)
         except Empty:
             self.timed_out()
@@ -73,10 +68,9 @@ class QueuedThread(NamedThread, ABC):
                 self._check_item()
             except BreakLoopError:
                 break
-            except Exception as exc:
-                LOG.error(f"{self.NAME} crashed:")
-                LOG.exception(exc)
-        LOG.verbose(f"Stopped {self.NAME} thread")
+            except Exception:
+                self.log.exception(f"{self.__class__.__name__} crashed:")
+        self.log.debug(f"Stopped {self.__class__.__name__}")
 
     def stop(self):
         """Stop the thread."""
@@ -84,16 +78,16 @@ class QueuedThread(NamedThread, ABC):
 
     def join(self):
         """End the thread."""
-        LOG.debug(f"Waiting for {self.NAME} to join.")
+        self.log.debug(f"Waiting for {self.__class__.__name__} to join.")
         super().join(self.SHUTDOWN_TIMEOUT)
-        LOG.debug(f"{self.NAME} joined.")
+        self.log.debug(f"{self.__class__.__name__} joined.")
 
 
 class AggregateMessageQueuedThread(QueuedThread, ABC):
     """Abstract Thread worker for buffering and aggregating messages."""
 
-    FLOOD_DELAY = 1
-    MAX_DELAY = 5
+    FLOOD_DELAY = 1.0
+    MAX_DELAY = 5.0
 
     def __init__(self, *args, **kwargs):
         """Initialize the cache."""
@@ -108,12 +102,12 @@ class AggregateMessageQueuedThread(QueuedThread, ABC):
     @abstractmethod
     def aggregate_items(self, item):
         """Abstract method for aggregating items."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @abstractmethod
     def send_all_items(self):
         """Abstract method for sending all items."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def cleanup_cache(self, keys):
         """Remove sent messages from the cache and record send times."""
