@@ -1,21 +1,20 @@
 """Test models."""
 import datetime
+import shutil
+from pathlib import Path
 
 from django.test import TestCase
 
-from codex.models import Comic
-from codex.models import Imprint
-from codex.models import Library
-from codex.models import Publisher
-from codex.models import Series
-from codex.models import Volume
+from codex.models import Comic, Imprint, Library, Publisher, Series, Volume
+
+TMP_DIR = Path("/tmp/codex.tests")  # noqa S108
 
 
 class ComicTestCase(TestCase):
     """Test Comic model."""
 
-    COMIC_PATH = "/comics/foo.cbz"
-    TITLE = "foo"
+    COMIC_PATH = TMP_DIR / "foo.cbz"
+    NAME = "foo"
     DECADE = 1970
     YEAR = 1975
     MONTH = 4
@@ -24,7 +23,9 @@ class ComicTestCase(TestCase):
 
     def setUp(self):
         """Set up for tests."""
-        library = Library.objects.create(path="/comics")
+        TMP_DIR.mkdir(exist_ok=True, parents=True)
+        self.COMIC_PATH.touch()
+        library = Library.objects.create(path=str(self.COMIC_PATH))
         publisher = Publisher.objects.create(name="FooPub")
         imprint = Imprint.objects.create(name="BarComics", publisher=publisher)
         series = Series.objects.create(
@@ -37,7 +38,7 @@ class ComicTestCase(TestCase):
             library=library,
             path=self.COMIC_PATH,
             issue=1,
-            title=ComicTestCase.TITLE,
+            name=ComicTestCase.NAME,
             publisher=publisher,
             imprint=imprint,
             series=series,
@@ -48,9 +49,13 @@ class ComicTestCase(TestCase):
             day=self.DAY,
         )
 
+    def tearDown(self):
+        """Tear down tests."""
+        shutil.rmtree(TMP_DIR)
+
     def test_comic_save(self):
         """Test comic model save method."""
         comic = Comic.objects.get(path=self.COMIC_PATH)
-        self.assertEqual(comic.title, self.TITLE)
-        self.assertEqual(comic.decade, self.DECADE)
-        self.assertEqual(comic.date, self.DATE)
+        assert comic.name == self.NAME
+        assert comic.decade == self.DECADE
+        assert comic.date == self.DATE
